@@ -29,9 +29,7 @@
 
 #include <gdk/gdktypes.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 typedef struct _GdkRgbCmap GdkRgbCmap;
 
@@ -137,11 +135,10 @@ void gdk_rgb_set_min_colors (gint     min_colors);
 GdkColormap *gdk_rgb_get_colormap (void);
 GdkVisual *  gdk_rgb_get_visual   (void);
 gboolean     gdk_rgb_ditherable   (void);
+gboolean     gdk_rgb_colormap_ditherable (GdkColormap *cmap);
 #endif
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 
 #endif /* __GDK_RGB_H__ */
