@@ -115,6 +115,9 @@ gdk_window_impl_quartz_finalize (GObject *object)
   if (impl->paint_clip_region)
     gdk_region_destroy (impl->paint_clip_region);
 
+  if (impl->transient_for)
+    g_object_unref (impl->transient_for);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -523,6 +526,9 @@ gdk_window_new (GdkWindow     *parent,
 
   private->parent = (GdkWindowObject *)parent;
 
+  private->accept_focus = TRUE;
+  private->focus_on_map = TRUE;
+
   if (attributes_mask & GDK_WA_X)
     private->x = attributes->x;
   else
@@ -778,12 +784,16 @@ all_parents_shown (GdkWindowObject *private)
   return FALSE;
 }
 
+/* Note: the raise argument is not really used, it doesn't seem
+ * possible to show a window without raising it?
+ */
 static void
 show_window_internal (GdkWindow *window,
 		      gboolean   raise)
 {
   GdkWindowObject *private;
   GdkWindowImplQuartz *impl;
+  gboolean focus_on_map;
 
   if (GDK_WINDOW_DESTROYED (window))
     return;
@@ -793,9 +803,22 @@ show_window_internal (GdkWindow *window,
   private = (GdkWindowObject *)window;
   impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
 
+  if (!GDK_WINDOW_IS_MAPPED (window))
+    focus_on_map = private->focus_on_map;
+  else
+    focus_on_map = TRUE;
+
   if (impl->toplevel)
     {
-      [impl->toplevel orderFront:nil];
+      /* We should make the window not raise for !raise, but at least
+       * this will keep it from getting focused in that case.
+       */
+      if (private->accept_focus && focus_on_map && raise &&
+          private->window_type != GDK_WINDOW_TEMP)
+        [impl->toplevel makeKeyAndOrderFront:impl->toplevel];
+      else
+        [impl->toplevel orderFront:nil];
+
       [impl->view setNeedsDisplay:YES];
     }
   else
@@ -815,6 +838,14 @@ show_window_internal (GdkWindow *window,
   if (private->state & GDK_WINDOW_STATE_ICONIFIED)
     gdk_window_iconify (window);
 
+  if (impl->transient_for && !GDK_WINDOW_DESTROYED (impl->transient_for))
+    {
+      GdkWindowImplQuartz *parent_impl;
+
+      parent_impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (impl->transient_for)->impl);
+      [parent_impl->toplevel addChildWindow:impl->toplevel ordered:NSWindowAbove];
+    }
+
   GDK_QUARTZ_RELEASE_POOL;
 }
 
@@ -856,6 +887,20 @@ gdk_window_hide (GdkWindow *window)
 
   if (impl->toplevel) 
     {
+      /* We must unset the transient while it is hidden, otherwise
+       * quartz won't hide the window.
+       */
+      if (impl->transient_for)
+        {
+          if (!GDK_WINDOW_DESTROYED (impl->transient_for))
+            {
+              GdkWindowImplQuartz *parent_impl;
+
+              parent_impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (impl->transient_for)->impl);
+              [parent_impl->toplevel removeChildWindow:impl->toplevel];
+            }
+        }
+
       [impl->toplevel orderOut:nil];
     }
   else if (impl->view)
@@ -1008,16 +1053,36 @@ void
 gdk_window_raise (GdkWindow *window)
 {
   g_return_if_fail (GDK_IS_WINDOW (window));
-  
-  /* FIXME: Implement */
+
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  /* FIXME: Only supported for toplevels currently. */
+  if (WINDOW_IS_TOPLEVEL (window))
+    {
+      GdkWindowImplQuartz *impl;
+
+      impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (window)->impl);
+      [impl->toplevel orderFront:impl->toplevel];
+    }
 }
 
 void
 gdk_window_lower (GdkWindow *window)
 {
   g_return_if_fail (GDK_IS_WINDOW (window));
-  
-  /* FIXME: Implement */
+
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  /* FIXME: Only supported for toplevels currently. */
+  if (WINDOW_IS_TOPLEVEL (window))
+    {
+      GdkWindowImplQuartz *impl;
+
+      impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (window)->impl);
+      [impl->toplevel orderBack:impl->toplevel];
+    }
 }
 
 void
@@ -1030,7 +1095,7 @@ gdk_window_set_background (GdkWindow      *window,
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   if (GDK_WINDOW_DESTROYED (window))
-      return;
+    return;
 
   impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
 
@@ -1481,7 +1546,56 @@ void
 gdk_window_set_transient_for (GdkWindow *window, 
 			      GdkWindow *parent)
 {
-  /* FIXME: Implement */
+  GdkWindowImplQuartz *window_impl;
+  GdkWindowImplQuartz *parent_impl;
+
+  if (GDK_WINDOW_DESTROYED (window) || GDK_WINDOW_DESTROYED (parent))
+    return;
+
+  window_impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (window)->impl);
+  if (!window_impl->toplevel)
+    return;
+
+  GDK_QUARTZ_ALLOC_POOL;
+
+  if (window_impl->transient_for)
+    {
+      if (!GDK_WINDOW_DESTROYED (window_impl->transient_for))
+        {
+          parent_impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (window_impl->transient_for)->impl);
+          [parent_impl->toplevel removeChildWindow:window_impl->toplevel];
+        }
+
+      g_object_unref (window_impl->transient_for);
+      window_impl->transient_for = NULL;
+    }
+
+  parent_impl = GDK_WINDOW_IMPL_QUARTZ (GDK_WINDOW_OBJECT (parent)->impl);
+  if (parent_impl->toplevel)
+    {
+      /* We save the parent because it needs to be unset/reset when
+       * hiding and showing the window. 
+       */
+
+      /* We don't set transients for tooltips, they are already
+       * handled by the window level being the top one. If we do, then
+       * the parent window will be brought to the top just because the
+       * tooltip is, which is not what we want.
+       */
+      if (gdk_window_get_type_hint (window) != GDK_WINDOW_TYPE_HINT_TOOLTIP)
+        {
+          window_impl->transient_for = g_object_ref (parent);
+
+          /* We only add the window if it is shown, otherwise it will
+           * be shown unconditionally here. If it is not shown, the
+           * window will be added in show() instead.
+           */
+          if (!(GDK_WINDOW_OBJECT (window)->state & GDK_WINDOW_STATE_WITHDRAWN))
+            [parent_impl->toplevel addChildWindow:window_impl->toplevel ordered:NSWindowAbove];
+        }
+    }
+  
+  GDK_QUARTZ_RELEASE_POOL;
 }
 
 void
@@ -1538,7 +1652,13 @@ void
 gdk_window_set_accept_focus (GdkWindow *window,
 			     gboolean accept_focus)
 {
-  /* FIXME: Implement */
+  GdkWindowObject *private;
+
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  private = (GdkWindowObject *)window;  
+
+  private->accept_focus = accept_focus != FALSE;
 }
 
 void
@@ -1575,7 +1695,13 @@ void
 gdk_window_set_focus_on_map (GdkWindow *window,
 			     gboolean focus_on_map)
 {
-  /* FIXME: Implement */
+  GdkWindowObject *private;
+
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  private = (GdkWindowObject *)window;  
+  
+  private->focus_on_map = focus_on_map != FALSE;
 }
 
 void          
