@@ -17,11 +17,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <stdlib.h>
 #include "gtkcellrenderertoggle.h"
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
+#include "gtkprivate.h"
 #include "gtktreeprivate.h"
+#include "gtkalias.h"
 
 static void gtk_cell_renderer_toggle_get_property  (GObject                    *object,
 						    guint                       param_id,
@@ -31,8 +34,6 @@ static void gtk_cell_renderer_toggle_set_property  (GObject                    *
 						    guint                       param_id,
 						    const GValue               *value,
 						    GParamSpec                 *pspec);
-static void gtk_cell_renderer_toggle_init       (GtkCellRendererToggle      *celltext);
-static void gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class);
 static void gtk_cell_renderer_toggle_get_size   (GtkCellRenderer            *cell,
 						 GtkWidget                  *widget,
  						 GdkRectangle               *cell_area,
@@ -66,7 +67,8 @@ enum {
   PROP_ACTIVATABLE,
   PROP_ACTIVE,
   PROP_RADIO,
-  PROP_INCONSISTENT
+  PROP_INCONSISTENT,
+  PROP_INDICATOR_SIZE
 };
 
 #define TOGGLE_WIDTH 12
@@ -78,47 +80,31 @@ static guint toggle_cell_signals[LAST_SIGNAL] = { 0 };
 typedef struct _GtkCellRendererTogglePrivate GtkCellRendererTogglePrivate;
 struct _GtkCellRendererTogglePrivate
 {
+  gint indicator_size;
+
   guint inconsistent : 1;
 };
 
 
-GType
-gtk_cell_renderer_toggle_get_type (void)
-{
-  static GType cell_toggle_type = 0;
-
-  if (!cell_toggle_type)
-    {
-      static const GTypeInfo cell_toggle_info =
-      {
-	sizeof (GtkCellRendererToggleClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_cell_renderer_toggle_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkCellRendererToggle),
-	0,              /* n_preallocs */
-	(GInstanceInitFunc) gtk_cell_renderer_toggle_init,
-      };
-
-      cell_toggle_type =
-	g_type_register_static (GTK_TYPE_CELL_RENDERER, "GtkCellRendererToggle",
-				&cell_toggle_info, 0);
-    }
-
-  return cell_toggle_type;
-}
+G_DEFINE_TYPE (GtkCellRendererToggle, gtk_cell_renderer_toggle, GTK_TYPE_CELL_RENDERER)
 
 static void
 gtk_cell_renderer_toggle_init (GtkCellRendererToggle *celltoggle)
 {
+  GtkCellRendererTogglePrivate *priv;
+
+  priv = GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE (celltoggle);
+
   celltoggle->activatable = TRUE;
   celltoggle->active = FALSE;
   celltoggle->radio = FALSE;
+
   GTK_CELL_RENDERER (celltoggle)->mode = GTK_CELL_RENDERER_MODE_ACTIVATABLE;
   GTK_CELL_RENDERER (celltoggle)->xpad = 2;
   GTK_CELL_RENDERER (celltoggle)->ypad = 2;
+
+  priv->indicator_size = TOGGLE_WIDTH;
+  priv->inconsistent = FALSE;
 }
 
 static void
@@ -140,8 +126,7 @@ gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class)
 							 P_("Toggle state"),
 							 P_("The toggle state of the button"),
 							 FALSE,
-							 G_PARAM_READABLE |
-							 G_PARAM_WRITABLE));
+							 GTK_PARAM_READWRITE));
 
   g_object_class_install_property (object_class,
 		                   PROP_INCONSISTENT,
@@ -149,8 +134,7 @@ gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class)
 					                 P_("Inconsistent state"),
 							 P_("The inconsistent state of the button"),
 							 FALSE,
-							 G_PARAM_READABLE |
-							 G_PARAM_WRITABLE));
+							 GTK_PARAM_READWRITE));
   
   g_object_class_install_property (object_class,
 				   PROP_ACTIVATABLE,
@@ -158,8 +142,7 @@ gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class)
 							 P_("Activatable"),
 							 P_("The toggle button can be activated"),
 							 TRUE,
-							 G_PARAM_READABLE |
-							 G_PARAM_WRITABLE));
+							 GTK_PARAM_READWRITE));
 
   g_object_class_install_property (object_class,
 				   PROP_RADIO,
@@ -167,12 +150,29 @@ gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class)
 							 P_("Radio state"),
 							 P_("Draw the toggle button as a radio button"),
 							 FALSE,
-							 G_PARAM_READABLE |
-							 G_PARAM_WRITABLE));
+							 GTK_PARAM_READWRITE));
 
+  g_object_class_install_property (object_class,
+				   PROP_INDICATOR_SIZE,
+				   g_param_spec_int ("indicator-size",
+						     P_("Indicator size"),
+						     P_("Size of check or radio indicator"),
+						     0,
+						     G_MAXINT,
+						     TOGGLE_WIDTH,
+						     GTK_PARAM_READWRITE));
 
+  
+  /**
+   * GtkCellRendererToggle::toggled:
+   * @cell_renderer: the object which received the signal
+   * @path: string representation of #GtkTreePath describing the 
+   *        event location
+   *
+   * The ::toggled signal is emitted when the cell is toggled. 
+   **/
   toggle_cell_signals[TOGGLED] =
-    g_signal_new ("toggled",
+    g_signal_new (I_("toggled"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkCellRendererToggleClass, toggled),
@@ -209,6 +209,9 @@ gtk_cell_renderer_toggle_get_property (GObject     *object,
     case PROP_RADIO:
       g_value_set_boolean (value, celltoggle->radio);
       break;
+    case PROP_INDICATOR_SIZE:
+      g_value_set_int (value, priv->indicator_size);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
@@ -226,24 +229,23 @@ gtk_cell_renderer_toggle_set_property (GObject      *object,
   GtkCellRendererTogglePrivate *priv;
 
   priv = GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE (object);
-  
+
   switch (param_id)
     {
     case PROP_ACTIVE:
       celltoggle->active = g_value_get_boolean (value);
-      g_object_notify (G_OBJECT(object), "active");
       break;
     case PROP_INCONSISTENT:
       priv->inconsistent = g_value_get_boolean (value);
-      g_object_notify (G_OBJECT (object), "inconsistent");
       break;
     case PROP_ACTIVATABLE:
       celltoggle->activatable = g_value_get_boolean (value);
-      g_object_notify (G_OBJECT(object), "activatable");
       break;
     case PROP_RADIO:
       celltoggle->radio = g_value_get_boolean (value);
-      g_object_notify (G_OBJECT(object), "radio");
+      break;
+    case PROP_INDICATOR_SIZE:
+      priv->indicator_size = g_value_get_int (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -253,7 +255,7 @@ gtk_cell_renderer_toggle_set_property (GObject      *object,
 
 /**
  * gtk_cell_renderer_toggle_new:
- * 
+ *
  * Creates a new #GtkCellRendererToggle. Adjust rendering
  * parameters using object properties. Object properties can be set
  * globally (with g_object_set()). Also, with #GtkTreeViewColumn, you
@@ -261,7 +263,7 @@ gtk_cell_renderer_toggle_set_property (GObject      *object,
  * can bind the "active" property on the cell renderer to a boolean value
  * in the model, thus causing the check button to reflect the state of
  * the model.
- * 
+ *
  * Return value: the new cell renderer
  **/
 GtkCellRenderer *
@@ -281,9 +283,12 @@ gtk_cell_renderer_toggle_get_size (GtkCellRenderer *cell,
 {
   gint calc_width;
   gint calc_height;
+  GtkCellRendererTogglePrivate *priv;
 
-  calc_width = (gint) cell->xpad * 2 + TOGGLE_WIDTH;
-  calc_height = (gint) cell->ypad * 2 + TOGGLE_WIDTH;
+  priv = GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE (cell);
+
+  calc_width = (gint) cell->xpad * 2 + priv->indicator_size;
+  calc_height = (gint) cell->ypad * 2 + priv->indicator_size;
 
   if (width)
     *width = calc_width;
@@ -305,6 +310,11 @@ gtk_cell_renderer_toggle_get_size (GtkCellRenderer *cell,
 	  *y_offset = MAX (*y_offset, 0);
 	}
     }
+  else
+    {
+      if (x_offset) *x_offset = 0;
+      if (y_offset) *y_offset = 0;
+    }
 }
 
 static void
@@ -339,7 +349,11 @@ gtk_cell_renderer_toggle_render (GtkCellRenderer      *cell,
   else
     shadow = celltoggle->active ? GTK_SHADOW_IN : GTK_SHADOW_OUT;
 
-  if ((flags & GTK_CELL_RENDERER_SELECTED) == GTK_CELL_RENDERER_SELECTED)
+  if (!cell->sensitive)
+    {
+      state = GTK_STATE_INSENSITIVE;
+    }
+  else if ((flags & GTK_CELL_RENDERER_SELECTED) == GTK_CELL_RENDERER_SELECTED)
     {
       if (GTK_WIDGET_HAS_FOCUS (widget))
 	state = GTK_STATE_SELECTED;
@@ -423,7 +437,7 @@ gtk_cell_renderer_toggle_set_radio (GtkCellRendererToggle *toggle,
  * gtk_cell_renderer_toggle_get_radio:
  * @toggle: a #GtkCellRendererToggle
  *
- * Returns wether we're rendering radio toggles rather than checkboxes. 
+ * Returns whether we're rendering radio toggles rather than checkboxes. 
  * 
  * Return value: %TRUE if we're rendering radio toggles rather than checkboxes
  **/
@@ -467,3 +481,6 @@ gtk_cell_renderer_toggle_set_active (GtkCellRendererToggle *toggle,
 
   g_object_set (toggle, "active", setting ? TRUE : FALSE, NULL);
 }
+
+#define __GTK_CELL_RENDERER_TOGGLE_C__
+#include "gtkaliasdef.c"
