@@ -25,6 +25,9 @@
 #include "gtkfilesystem.h"
 #include "gtkfilesystemmodel.h"
 #include "gtkliststore.h"
+#include "gtkrecentmanager.h"
+#include "gtksearchengine.h"
+#include "gtkquery.h"
 #include "gtktooltips.h"
 #include "gtktreemodelsort.h"
 #include "gtktreestore.h"
@@ -109,10 +112,6 @@ struct _GtkFileChooserDialogPrivate
   char *file_system;
 
   /* for use with GtkFileChooserEmbed */
-  gint default_width;
-  gint default_height;
-  gboolean resize_horizontally;
-  gboolean resize_vertically;
   gboolean response_requested;
 };
 
@@ -147,6 +146,12 @@ typedef enum {
   LOCATION_MODE_FILENAME_ENTRY
 } LocationMode;
 
+typedef enum {
+  OPERATION_MODE_BROWSE,
+  OPERATION_MODE_SEARCH,
+  OPERATION_MODE_RECENT
+} OperationMode;
+
 struct _GtkFileChooserDefault
 {
   GtkVBox parent_instance;
@@ -175,11 +180,28 @@ struct _GtkFileChooserDefault
   GtkWidget *browse_files_popup_menu_add_shortcut_item;
   GtkWidget *browse_files_popup_menu_hidden_files_item;
   GtkWidget *browse_new_folder_button;
+  GtkWidget *browse_path_bar_hbox;
   GtkWidget *browse_path_bar;
 
   GtkFileSystemModel *browse_files_model;
   char *browse_files_last_selected_name;
 
+  /* OPERATION_MODE_SEARCH */
+  GtkWidget *search_hbox;
+  GtkWidget *search_entry;
+  GtkSearchEngine *search_engine;
+  GtkQuery *search_query;
+  GtkListStore *search_model;
+  GtkTreeModelFilter *search_model_filter;
+  GtkTreeModelSort *search_model_sort;
+
+  /* OPERATION_MODE_RECENT */
+  GtkRecentManager *recent_manager;
+  GtkListStore *recent_model;
+  guint load_recent_id;
+  GtkTreeModelFilter *recent_model_filter;
+  GtkTreeModelSort *recent_model_sort;
+
   GtkWidget *filter_combo_hbox;
   GtkWidget *filter_combo;
   GtkWidget *preview_box;
@@ -195,7 +217,16 @@ struct _GtkFileChooserDefault
   LocationMode location_mode;
 
   GtkListStore *shortcuts_model;
-  GtkTreeModel *shortcuts_filter_model;
+
+  /* Filter for the shortcuts pane.  We filter out the "current folder" row and
+   * the separator that we use for the "Save in folder" combo.
+   */
+  GtkTreeModel *shortcuts_pane_filter_model;
+  
+  /* Filter for the "Save in folder" combo.  We filter out the Search row and
+   * its separator.
+   */
+  GtkTreeModel *shortcuts_combo_filter_model;
 
   GtkTreeModelSort *sort_model;
 
@@ -206,6 +237,7 @@ struct _GtkFileChooserDefault
   GtkFileSystemHandle *update_current_folder_handle;
   GtkFileSystemHandle *show_and_select_paths_handle;
   GtkFileSystemHandle *should_respond_get_info_handle;
+  GtkFileSystemHandle *file_exists_get_info_handle;
   GtkFileSystemHandle *update_from_entry_handle;
   GtkFileSystemHandle *shortcuts_activate_iter_handle;
   GSList *pending_handles;
@@ -214,6 +246,8 @@ struct _GtkFileChooserDefault
   ReloadState reload_state;
   guint load_timeout_id;
 
+  OperationMode operation_mode;
+
   GSList *pending_select_paths;
 
   GtkFileFilter *current_filter;
@@ -221,9 +255,6 @@ struct _GtkFileChooserDefault
 
   GtkTooltips *tooltips;
 
-  gboolean has_home;
-  gboolean has_desktop;
-
   int num_volumes;
   int num_shortcuts;
   int num_bookmarks;
@@ -238,6 +269,7 @@ struct _GtkFileChooserDefault
 
   GtkTreeViewColumn *list_name_column;
   GtkCellRenderer *list_name_renderer;
+  GtkTreeViewColumn *list_mtime_column;
 
   GSource *edited_idle;
   char *edited_new_text;
@@ -253,6 +285,9 @@ struct _GtkFileChooserDefault
   GSource *shortcuts_drag_outside_idle;
 #endif
 
+  gint default_width;
+  gint default_height;
+
   /* Flags */
 
   guint local_only : 1;
@@ -264,6 +299,11 @@ struct _GtkFileChooserDefault
   guint list_sort_ascending : 1;
   guint changing_folder : 1;
   guint shortcuts_current_folder_active : 1;
+  guint expand_folders : 1;
+  guint has_home : 1;
+  guint has_desktop : 1;
+  guint has_search : 1;
+  guint has_recent : 1;
 
 #if 0
   guint shortcuts_drag_outside : 1;
