@@ -29,20 +29,22 @@
 
 #include	<config.h>
 
+#undef GTK_DISABLE_DEPRECATED
 #include	"gtkitemfactory.h"
-#include	"gtk/gtkmenubar.h"
-#include	"gtk/gtkmenu.h"
-#include	"gtk/gtkmenuitem.h"
-#include	"gtk/gtkradiomenuitem.h"
-#include	"gtk/gtkcheckmenuitem.h"
-#include	"gtk/gtkimagemenuitem.h"
-#include	"gtk/gtktearoffmenuitem.h"
-#include	"gtk/gtkaccelmap.h"
-#include	"gtk/gtkaccellabel.h"
+#include	"gtkoptionmenu.h"
+#include	"gtkmenubar.h"
+#include	"gtkmenu.h"
+#include	"gtkmenuitem.h"
+#include	"gtkradiomenuitem.h"
+#include	"gtkcheckmenuitem.h"
+#include	"gtkimagemenuitem.h"
+#include	"gtktearoffmenuitem.h"
+#include	"gtkaccelmap.h"
+#include	"gtkaccellabel.h"
 #include        "gdk/gdkkeysyms.h"
-#include	"gtk/gtkimage.h"
-#include	"gtk/gtkstock.h"
-#include	"gtk/gtkiconfactory.h"
+#include	"gtkimage.h"
+#include	"gtkstock.h"
+#include	"gtkiconfactory.h"
 #include	"gtkintl.h"
 #include	<string.h>
 #include	<fcntl.h>
@@ -51,10 +53,7 @@
 #endif
 #include	<stdio.h>
 
-#undef GTK_DISABLE_DEPRECATED
-#include	"gtk/gtkoptionmenu.h"
-#define GTK_DISABLE_DEPRECATED
-#include 	"gtkalias.h"
+#include "gtkalias.h"
 
 /* --- defines --- */
 #define		ITEM_FACTORY_STRING	((gchar*) item_factory_string)
@@ -98,7 +97,7 @@ static GQuark		 quark_type_separator_item = 0;
 static GQuark		 quark_type_branch = 0;
 static GQuark		 quark_type_last_branch = 0;
 
-G_DEFINE_TYPE (GtkItemFactory, gtk_item_factory, GTK_TYPE_OBJECT);
+G_DEFINE_TYPE (GtkItemFactory, gtk_item_factory, GTK_TYPE_OBJECT)
 
 /* --- functions --- */
 static void
