@@ -28,12 +28,10 @@
 #define __GTK_TOOLTIPS_H__
 
 #include <gtk/gtkwidget.h>
+#include <gtk/gtkwindow.h>
 
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 #define GTK_TYPE_TOOLTIPS                  (gtk_tooltips_get_type ())
 #define GTK_TOOLTIPS(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_TOOLTIPS, GtkTooltips))
@@ -59,10 +57,11 @@ struct _GtkTooltips
 {
   GtkObject parent_instance;
 
+  /*< private >*/
   GtkWidget *tip_window;
   GtkWidget *tip_label;
   GtkTooltipsData *active_tips_data;
-  GList *tips_data_list;
+  GList *_tips_data_list; /* unused */
 
   guint   delay : 30;
   guint	  enabled : 1;
@@ -102,9 +101,10 @@ void             gtk_tooltips_force_window (GtkTooltips   *tooltips);
 
 void             _gtk_tooltips_toggle_keyboard_mode (GtkWidget *widget);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+gboolean         gtk_tooltips_get_info_from_tip_window (GtkWindow    *tip_window,
+                                                        GtkTooltips **tooltips,
+                                                        GtkWidget   **current_widget);
 
+G_END_DECLS
 
 #endif /* __GTK_TOOLTIPS_H__ */
